package com.qmth.ops.api.controller.ai;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.core.ai.model.AiConstants;
import com.qmth.boot.core.ai.model.llm.*;
import com.qmth.boot.core.exception.ForbiddenException;
import com.qmth.boot.core.exception.NotFoundException;
import com.qmth.boot.tools.freemarker.FreemarkerUtil;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.ops.api.security.AccessOrg;
import com.qmth.ops.biz.domain.LlmOrgConfig;
import com.qmth.ops.biz.domain.LlmPromptTemplate;
import com.qmth.ops.biz.service.LlmClientService;
import com.qmth.ops.biz.service.LlmOrgConfigService;
import com.qmth.ops.biz.service.LlmPromptTemplateService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@Aac(auth = true, signType = SignatureType.SECRET)
public class LlmController {

    @Resource
    private LlmOrgConfigService llmOrgConfigService;

    @Resource
    private LlmClientService llmClientService;

    @Resource
    private LlmPromptTemplateService llmPromptTemplateService;

    @PostMapping(AiConstants.LLM_CHAT_PATH)
    public ChatResult chat(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type,
            @RequestBody @Validated ChatRequest request) throws Exception {
        LlmOrgConfig config = llmOrgConfigService.findByOrgAndAppType(accessOrg.getOrg().getId(), type);
        if (config == null || config.getLeftCount() <= 0) {
            throw new ForbiddenException(
                    "Chat api is disabled or exhausted for org=" + accessOrg.getOrg().getCode() + ", app_type=" + type);
        }
        ChatResult result = llmClientService.chat(request, config.getModelId(), type);
        llmOrgConfigService.consume(config);
        return result;
    }

    @PostMapping(AiConstants.LLM_CHAT_TEMPLATE_PATH)
    public ChatResult chatTemplate(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type, @RequestBody Object param)
            throws Exception {
        LlmOrgConfig config = llmOrgConfigService.findByOrgAndAppType(accessOrg.getOrg().getId(), type);
        if (config == null || config.getLeftCount() <= 0) {
            throw new ForbiddenException(
                    "Chat api is disabled or exhausted for org=" + accessOrg.getOrg().getCode() + ", app_type=" + type);
        }
        LlmPromptTemplate llmPromptTemplate = llmPromptTemplateService.findById(config.getPromptId());
        if (llmPromptTemplate == null) {
            throw new NotFoundException(
                    "Chat prompt template not found for app_type=" + type + ", modelId=" + config.getModelId());
        }
        ChatRequest request = new ChatRequest();
        String systemMessage = FreemarkerUtil.getValue(llmPromptTemplate.getSystem(), param, null);
        String userMessage = FreemarkerUtil.getValue(llmPromptTemplate.getUser(), param, null);
        if (StringUtils.isNotBlank(systemMessage)) {
            request.addMessage(ChatRole.system, systemMessage);
        }
        if (StringUtils.isNotBlank(userMessage)) {
            request.addMessage(ChatRole.user, userMessage);
        }
        ChatResult result = llmClientService.chat(request, config.getModelId(), type);
        llmOrgConfigService.consume(config);
        return result;
    }

    @PostMapping(AiConstants.LLM_BALANCE_PATH)
    public LlmAppBalance balance(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type) {
        LlmAppBalance balance = new LlmAppBalance();
        LlmOrgConfig config = llmOrgConfigService.findByOrgAndAppType(accessOrg.getOrg().getId(), type);
        if (config != null) {
            balance.setPermitCount(config.getPermitCount());
            balance.setLeftCount(config.getLeftCount());
        }
        return balance;
    }

    @PostMapping(AiConstants.LLM_PROMPT_TEMPLATE_PATH)
    public PromptTemplate getPromptTemplate(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type) {
        PromptTemplate template = new PromptTemplate();
        LlmOrgConfig config = llmOrgConfigService.findByOrgAndAppType(accessOrg.getOrg().getId(), type);
        if (config != null) {
            LlmPromptTemplate llmPromptTemplate = llmPromptTemplateService.findById(config.getPromptId());
            if (llmPromptTemplate != null) {
                template.setSystem(llmPromptTemplate.getSystem());
                template.setUser(llmPromptTemplate.getUser());
            }
        }
        return template;
    }
}
