package com.qmth.ops.biz.ai.client.baidu.ocr;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.ocr.OcrType;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.ReentrantException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.exception.UnauthorizedException;
import com.qmth.boot.tools.codec.CodecUtils;
import com.qmth.boot.tools.models.ByteArray;
import com.qmth.ops.biz.ai.client.OcrApiClient;
import com.qmth.ops.biz.ai.client.OcrApiConfig;
import com.qmth.ops.biz.ai.client.baidu.BaiduError;
import com.qmth.ops.biz.ai.client.baidu.BceV1Signer;
import okhttp3.FormBody;
import okhttp3.HttpUrl;
import okhttp3.Interceptor;
import okhttp3.Request;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;
import java.util.stream.Collectors;

public class BaiduOcrClient extends OcrApiClient {

    public BaiduOcrClient(OcrApiConfig config) {
        super(config);
    }

    @Override
    protected String buildResult(byte[] data, ObjectMapper mapper) throws IOException {
        BaiduOcrResult result = mapper.readValue(data, BaiduOcrResult.class);
        if (result != null && result.getWordsList() != null) {
            if (!result.getWordsList().isEmpty()) {
                return mapper.readValue(data, BaiduOcrResult.class).getWordsList().stream().map(BaiduOcrWords::getWords)
                        .collect(Collectors.joining("\n")).replaceAll("☰", "").replaceAll("≡", "");
            } else {
                return StringUtils.EMPTY;
            }
        } else {
            return handleError(data, 500, mapper);
        }
    }

    @Override
    protected String handleError(byte[] data, int statusCode, ObjectMapper mapper) {
        BaiduError error = null;
        if (data != null) {
            try {
                error = mapper.readValue(data, BaiduError.class);
            } catch (Exception ignore) {
            }
        }
        switch (statusCode) {
        case 400:
        case 413:
            throw new ParameterException(error != null ? error.getMessage() : "ocr request parameter error");
        case 401:
            throw new UnauthorizedException(error != null ? error.getMessage() : "ocr api unauthorized");
        case 503:
        case 504:
            throw new ReentrantException(error != null ? error.getMessage() : "ocr api temporary faile");
        default:
            throw new StatusException(error != null ? error.getMessage() : "ocr api error");
        }
    }

    @Override
    protected Request buildRequest(OcrType type, byte[] image) throws Exception {
        String url = buildUrl(type);
        SimpleDateFormat format = new SimpleDateFormat(BceV1Signer.DATE_FORMAT_PATTERN);
        format.setTimeZone(TimeZone.getTimeZone("UTC"));
        return new Request.Builder().url(url).addHeader(BceV1Signer.X_BCE_DATE, format.format(new Date()))
                .addHeader(HttpHeaders.HOST, HttpUrl.parse(url).host())
                .addHeader(HttpHeaders.CONTENT_TYPE, "application/x-www-form-urlencoded").post(buildBody(type, image))
                .build();
    }

    private String buildUrl(OcrType type) {
        StringBuilder url = new StringBuilder(getConfig().getUrl());
        if (type == OcrType.GENERAL) {
            url.append("general_basic");
        } else if (type == OcrType.HANDWRITING) {
            url.append("handwriting");
        }
        return url.toString();
    }

    private FormBody buildBody(OcrType type, byte[] image) {
        FormBody.Builder builder = new FormBody.Builder();
        builder.add("image", CodecUtils.toBase64(image));
        if (type == OcrType.HANDWRITING) {
            builder.add("detect_alteration", "false");
        }
        return builder.build();
    }

    @Override
    protected Interceptor getInterceptor() {
        return chain -> {
            Request request = chain.request();
            return chain.proceed(request.newBuilder().addHeader(HttpHeaders.AUTHORIZATION,
                    BceV1Signer.sign(request, getConfig().getKey(), getConfig().getSecret())).build());
        };
    }

    public static void main(String[] args) throws Exception {
        OcrApiConfig config = new OcrApiConfig();
        config.setUrl("https://aip.baidubce.com/rest/2.0/ocr/v1/");
        config.setKey("");
        config.setSecret("");
        config.setQps(10);
        BaiduOcrClient client = new BaiduOcrClient(config);
        System.out.println(client.forImage(OcrType.HANDWRITING,
                ByteArray.fromFile(new File("/Users/luoshi/Downloads/test.jpg")).value()));
    }

}
